' 74HCT595 - L
' SHIFT_REG_CLOCK (11)          - P10
' STORE_REG_CLOCK (12)          - P11
' SERIAL_DATA_IN  (14)          - P12
'
' 74HCT595 - R
' SHIFT_REG_CLOCK (11)          - P13
' STORE_REG_CLOCK (12)          - P14
' SERIAL_DATA_IN  (14)          - P15
'
' DS1620 THERMOSTAT MODULE
' DQ              (01)          - P16
' CLK             (02)          - P17
' RST             (03)          - P18


CON
  _clkmode = xtal1 + pll16x        'Sets the clock speed to 80 MHz using a times 16 multiplier
  _xinfreq = 5_000_000             'Crystal speed: 5 MHz

  PIN_DQ                        = 16
  PIN_CLK                       = 17
  PIN_RST                       = 18

  PIN_SPIL_CLOCK                = 10
  PIN_SPIL_STORE                = 11
  PIN_SPIL_DATA                 = 12
  PIN_SPIR_CLOCK                = 13
  PIN_SPIR_STORE                = 14
  PIN_SPIR_DATA                 = 15

  CMD_READ_TEMP                 = $AA
  CMD_WRITE_TH                  = $01
  CMD_WRITE_TL                  = $02
  CMD_READ_TH                   = $A1
  CMD_READ_TL                   = $A2
  CMD_START_CONVERT             = $EE
  CMD_STOP_CONVERT              = $22                        
  CMD_WRITE_CONFIG              = $0C
  CMD_READ_CONFIG               = $AC


VAR
  long  lTemperature
  long  lShiftedTemp
  long  lFTemp
  long  lSubTemperature

  
OBJ
  IO            : "ShiftIO"
  TIME          : "Timing"
  DEBUG         : "FullDuplexSerial"
  SEGMENTLED    : "7SegmentLED"


PUB Main

  'TEST CODE FOR THE 7 SEGMENT LED OBJECT
  SEGMENTLED.SetValue (PIN_SPIL_DATA, PIN_SPIL_CLOCK, PIN_SPIL_STORE, -1)
  SEGMENTLED.SetValue (PIN_SPIR_DATA, PIN_SPIR_CLOCK, PIN_SPIR_STORE, -1)
  
  DEBUG.start (31, 30, 0, 9600)


  '=====================================
  'TEST THE DS1620
  '=====================================
  
  '--- Start: Init Code ---
  dira [PIN_DQ]~~
  dira [PIN_CLK]~~
  dira [PIN_RST]~~
  'SET CONFIGURATION TO CPU / FREE-RUN
  outa[PIN_RST]~~
  IO.shiftout (PIN_DQ, PIN_CLK, IO#LsbFirst, CMD_WRITE_CONFIG, 8)
  IO.shiftout (PIN_DQ, PIN_CLK, IO#LsbFirst, %10, 8)
  outa[PIN_RST]~
  TIME.pause1ms (5)
  outa[PIN_RST]~~
  'BEGIN TEMPERATURE SAMPLING
  IO.shiftout (PIN_DQ, PIN_CLK, io#LsbFirst, CMD_START_CONVERT, 8)
  outa[PIN_RST]~
  '--- End: Init Code ---

  'Try to read in the temp and dump the raw bin, shifted bin, and dec value of shifted bin
  outa[PIN_RST]~~
  IO.shiftout (PIN_DQ, PIN_CLK, io#LsbFirst, CMD_READ_TEMP, 8)
  lTemperature := IO.shiftin (PIN_DQ, PIN_CLK, io#LsbFirst, 9) 
  outa[PIN_RST]~

  'Dump the .5 bit
  lShiftedTemp := lTemperature >> 1
  'Convert to F
  lFTemp := lShiftedTemp * (9/5) + 32

  lSubTemperature := GetTemperature (PIN_DQ, PIN_CLK, PIN_RST)

  repeat
    DEBUG.str (string ("BIN RAW: "))
    DEBUG.bin (lTemperature, 9)
    DEBUG.tx (13)

    DEBUG.str (string ("DEC RAW: "))
    DEBUG.dec (lTemperature)
    DEBUG.tx (13)

    DEBUG.str (string ("BIN SFT: "))
    DEBUG.bin (lShiftedTemp, 9)
    DEBUG.tx (13)

    DEBUG.str (string ("DEC SFT (C): "))
    DEBUG.dec (lShiftedTemp)
    DEBUG.tx (13)

    DEBUG.str (string ("DEC SFT (F): "))
    DEBUG.dec (lFTemp)
    DEBUG.tx (13)

    DEBUG.str (string ("SUB TMP: "))
    DEBUG.dec (lSubTemperature)
    DEBUG.tx (13)

    TIME.pause1ms (3000)   


  '=====================================
  '=====================================


PUB InitDS1620 (DQ, CLK, RST)

  '--- Start: Init Code ---
  dira [DQ]~~
  dira [CLK]~~
  dira [RST]~~
  'SET CONFIGURATION TO CPU / FREE-RUN
  outa[RST]~~
  IO.shiftout (DQ, CLK, IO#LsbFirst, CMD_WRITE_CONFIG, 8)
  IO.shiftout (DQ, CLK, IO#LsbFirst, %10, 8)
  outa[RST]~
  TIME.pause1ms (5)
  outa[RST]~~
  'BEGIN TEMPERATURE SAMPLING
  IO.shiftout (DQ, CLK, io#LsbFirst, CMD_START_CONVERT, 8)
  outa[RST]~
  '--- End: Init Code ---


PUB GetTemperature (DQ, CLK, RST): lTemp | lReadTemp

  outa[RST]~~
  IO.shiftout (DQ, CLK, io#LsbFirst, CMD_READ_TEMP, 8)
  lReadTemp := IO.shiftin (DQ, CLK, io#LsbFirst, 9) 
  outa[RST]~

  '--- debug ---
  DEBUG.str (string ("BIN RAW:  "))
  DEBUG.bin (lReadTemp, 9)
  DEBUG.tx (13)
   
  DEBUG.str (string ("DEC RAW:  "))
  DEBUG.dec (lReadTemp)
  DEBUG.tx (13)
   
  lReadTemp := lReadTemp >> 1

  DEBUG.str (string ("BIN SFT:  "))
  DEBUG.bin (lReadTemp, 9)
  DEBUG.tx (13)
   
  DEBUG.str (string ("DEC SFT:  "))
  DEBUG.dec (lReadTemp)
  DEBUG.tx (13)
   
  lTemp := lReadTemp * (9/5) + 32

  DEBUG.str (string ("DEC CNV:  "))
  DEBUG.dec (lTemp)
  DEBUG.tx (13)

   